@@ -35,6 +35,7 @@
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include <gio/gio.h>
+#include <cairo-gobject.h>
 
 #define sure_string(s)                    ((const char *)((s)!=NULL?(s):""))
 #define DESKTOP_ENTRY_GROUP		  "Desktop Entry"
@@ -490,19 +491,29 @@ entry_changed_cb (GtkWidget *entry,
     }
 }
 
-static GdkPixbuf *
-get_pixbuf_for_icon (GIcon *icon)
+#define CAJA_OPEN_WITH_DIALOG_ICON_SIZE 24
+static cairo_surface_t *
+get_surface_for_icon (GIcon *icon)
 {
-    GdkPixbuf  *pixbuf;
+    cairo_surface_t  *surface;
     char *filename;
+    gint icon_scale;
+
+    surface = NULL;
+    icon_scale = gdk_window_get_scale_factor (gdk_get_default_root_window ());
 
-    pixbuf = NULL;
     if (G_IS_FILE_ICON (icon))
     {
         filename = g_file_get_path (g_file_icon_get_file (G_FILE_ICON (icon)));
         if (filename)
         {
-            pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 24, 24, NULL);
+            GdkPixbuf *pixbuf;
+            pixbuf = gdk_pixbuf_new_from_file_at_size (filename,
+                                                       CAJA_OPEN_WITH_DIALOG_ICON_SIZE * icon_scale,
+                                                       CAJA_OPEN_WITH_DIALOG_ICON_SIZE * icon_scale,
+                                                       NULL);
+            surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, icon_scale, NULL);
+            g_object_unref (pixbuf);
         }
         g_free (filename);
     }
@@ -525,22 +536,27 @@ get_pixbuf_for_icon (GIcon *icon)
             {
                 *p = 0;
             }
-            pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-                                               icon_no_extension, 24, 0, NULL);
+            surface = gtk_icon_theme_load_surface (gtk_icon_theme_get_default (),
+                                                   icon_no_extension,
+                                                   CAJA_OPEN_WITH_DIALOG_ICON_SIZE,
+                                                   icon_scale,
+                                                   NULL,
+                                                   GTK_ICON_LOOKUP_FORCE_SIZE,
+                                                   NULL);
             g_free (icon_no_extension);
         }
     }
-    return pixbuf;
+    return surface;
 }
 
 static gboolean
 caja_open_with_dialog_add_icon_idle (CajaOpenWithDialog *dialog)
 {
-    GtkTreeIter   iter;
-    GtkTreePath  *path;
-    GdkPixbuf    *pixbuf;
-    GIcon *icon;
-    gboolean      long_operation;
+    GtkTreeIter      iter;
+    GtkTreePath     *path;
+    cairo_surface_t *surface;
+    GIcon           *icon;
+    gboolean         long_operation;
 
     long_operation = FALSE;
     do
@@ -573,12 +589,12 @@ caja_open_with_dialog_add_icon_idle (CajaOpenWithDialog *dialog)
             continue;
         }
 
-        pixbuf = get_pixbuf_for_icon (icon);
-        if (pixbuf)
+        surface = get_surface_for_icon (icon);
+        if (surface)
         {
             long_operation = TRUE;
-            gtk_list_store_set (dialog->details->program_list_store, &iter, COLUMN_ICON, pixbuf, -1);
-            g_object_unref (pixbuf);
+            gtk_list_store_set (dialog->details->program_list_store, &iter, COLUMN_ICON, surface, -1);
+            cairo_surface_destroy (surface);
         }
 
         /* don't go back into the main loop if this wasn't very hard to do */
@@ -675,7 +691,7 @@ caja_open_with_dialog_add_items_idle (CajaOpenWithDialog *dialog)
     /* create list store */
     dialog->details->program_list_store = gtk_list_store_new (NUM_COLUMNS,
                                           G_TYPE_APP_INFO,
-                                          GDK_TYPE_PIXBUF,
+                                          CAIRO_GOBJECT_TYPE_SURFACE,
                                           G_TYPE_ICON,
                                           G_TYPE_STRING,
                                           G_TYPE_STRING,
@@ -723,7 +739,7 @@ caja_open_with_dialog_add_items_idle (CajaOpenWithDialog *dialog)
     column = gtk_tree_view_column_new ();
     gtk_tree_view_column_pack_start (column, renderer, FALSE);
     gtk_tree_view_column_set_attributes (column, renderer,
-                                         "pixbuf", COLUMN_ICON,
+                                         "surface", COLUMN_ICON,
                                          NULL);
 
     renderer = gtk_cell_renderer_text_new ();
@@ -851,11 +867,7 @@ caja_open_with_dialog_init (CajaOpenWithDialog *dialog)
     gtk_box_pack_start (GTK_BOX (vbox), vbox2, TRUE, TRUE, 0);
 
     dialog->details->label = gtk_label_new ("");
-#if GTK_CHECK_VERSION (3, 16, 0)
     gtk_label_set_xalign (GTK_LABEL (dialog->details->label), 0.0);
-#else
-    gtk_misc_set_alignment (GTK_MISC (dialog->details->label), 0.0, 0.5);
-#endif
     gtk_label_set_line_wrap (GTK_LABEL (dialog->details->label), TRUE);
     gtk_box_pack_start (GTK_BOX (vbox2), dialog->details->label,
                         FALSE, FALSE, 0);
@@ -870,6 +882,8 @@ caja_open_with_dialog_init (CajaOpenWithDialog *dialog)
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                     GTK_POLICY_AUTOMATIC,
                                     GTK_POLICY_AUTOMATIC);
+    gtk_scrolled_window_set_overlay_scrolling (GTK_SCROLLED_WINDOW (scrolled_window), FALSE);
+
     dialog->details->program_list = gtk_tree_view_new ();
     gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (dialog->details->program_list),
                                        FALSE);
@@ -878,12 +892,9 @@ caja_open_with_dialog_init (CajaOpenWithDialog *dialog)
     gtk_box_pack_start (GTK_BOX (vbox2), scrolled_window, TRUE, TRUE, 0);
 
     dialog->details->desc_label = gtk_label_new (_("Select an application to view its description."));
-#if GTK_CHECK_VERSION (3, 16, 0)
     gtk_label_set_xalign (GTK_LABEL (dialog->details->desc_label), 0.0);
-#else
-    gtk_misc_set_alignment (GTK_MISC (dialog->details->desc_label), 0.0, 0.5);
-#endif
     gtk_label_set_justify (GTK_LABEL (dialog->details->desc_label), GTK_JUSTIFY_LEFT);
+    gtk_label_set_max_width_chars (GTK_LABEL (dialog->details->desc_label), 54);
     gtk_label_set_line_wrap (GTK_LABEL (dialog->details->desc_label), TRUE);
     gtk_label_set_single_line_mode (GTK_LABEL (dialog->details->desc_label), FALSE);
     gtk_box_pack_start (GTK_BOX (vbox2), dialog->details->desc_label, FALSE, FALSE, 0);
@@ -964,7 +975,7 @@ caja_open_with_dialog_init (CajaOpenWithDialog *dialog)
     gtk_widget_show (label);
     dialog->details->open_label = label;
 
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, FALSE, 0);
 
     gtk_widget_show (dialog->details->button);
     gtk_widget_set_can_default (dialog->details->button, TRUE);
