@@ -39,6 +39,7 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
+#include <cairo-gobject.h>
 #include <libcaja-private/caja-clipboard.h>
 #include <libcaja-private/caja-clipboard-monitor.h>
 #include <libcaja-private/caja-desktop-icon-file.h>
@@ -48,12 +49,17 @@
 #include <libcaja-private/caja-file-utilities.h>
 #include <libcaja-private/caja-global-preferences.h>
 #include <libcaja-private/caja-icon-names.h>
+#include <libcaja-private/caja-mime-actions.h>
 #include <libcaja-private/caja-program-choosing.h>
 #include <libcaja-private/caja-tree-view-drag-dest.h>
 #include <libcaja-private/caja-sidebar-provider.h>
 #include <libcaja-private/caja-module.h>
 #include <libcaja-private/caja-window-info.h>
 #include <libcaja-private/caja-window-slot-info.h>
+#include <libcaja-private/caja-directory.h>
+#include <libcaja-private/caja-directory-private.h>
+#include <libcaja-private/caja-file.h>
+#include <libcaja-private/caja-file-private.h>
 
 typedef struct
 {
@@ -120,6 +126,7 @@ static void  fm_tree_view_activate_file     (FMTreeView *view,
         CajaFile *file,
         CajaWindowOpenFlags flags);
 static GType fm_tree_view_provider_get_type (void);
+static GtkWindow *fm_tree_view_get_containing_window (FMTreeView *view);
 
 static void create_popup_menu (FMTreeView *view);
 
@@ -926,7 +933,14 @@ static void
 fm_tree_view_open_in_new_window_cb (GtkWidget *menu_item,
                                     FMTreeView *view)
 {
-    fm_tree_view_activate_file (view, view->details->popup_file, CAJA_WINDOW_OPEN_FLAG_NEW_WINDOW);
+    /* fm_tree_view_activate_file (view, view->details->popup_file, CAJA_WINDOW_OPEN_FLAG_NEW_WINDOW); */
+
+    caja_mime_activate_file  (fm_tree_view_get_containing_window (view),
+                              caja_window_info_get_active_slot (view->details->window),
+                              view->details->popup_file,
+                              g_file_get_path (view->details->popup_file->details->directory->details->location),
+                              CAJA_WINDOW_OPEN_FLAG_NEW_WINDOW,
+                              0);
 }
 
 static void
@@ -1469,9 +1483,7 @@ create_tree (FMTreeView *view)
     cell = gtk_cell_renderer_pixbuf_new ();
     gtk_tree_view_column_pack_start (column, cell, FALSE);
     gtk_tree_view_column_set_attributes (column, cell,
-                                         "pixbuf", FM_TREE_MODEL_CLOSED_PIXBUF_COLUMN,
-                                         "pixbuf_expander_closed", FM_TREE_MODEL_CLOSED_PIXBUF_COLUMN,
-                                         "pixbuf_expander_open", FM_TREE_MODEL_OPEN_PIXBUF_COLUMN,
+                                         "surface", FM_TREE_MODEL_CLOSED_SURFACE_COLUMN,
                                          NULL);
 
     cell = gtk_cell_renderer_text_new ();
@@ -1578,6 +1590,7 @@ fm_tree_view_init (FMTreeView *view)
     gtk_scrolled_window_set_hadjustment (GTK_SCROLLED_WINDOW (view), NULL);
     gtk_scrolled_window_set_vadjustment (GTK_SCROLLED_WINDOW (view), NULL);
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (view), GTK_SHADOW_IN);
+    gtk_scrolled_window_set_overlay_scrolling (GTK_SCROLLED_WINDOW (view), FALSE);
 
     gtk_widget_show (GTK_WIDGET (view));
 
